from __future__ import division
from cStringIO import StringIO

from matplotlib.afm import AFM
from matplotlib.backend_bases import RendererBase, GraphicsContextBase,\
     FigureManagerBase, FigureCanvasBase

from matplotlib.cbook import True, False

from matplotlib.figure import Figure
from matplotlib.font_manager import fontManager
from matplotlib.ft2font import FT2Font
from matplotlib._matlab_helpers import Gcf
from matplotlib import rcParams

import sys,os,math

def error_msg_svg(msg, *args):
    print >>sys.stderr, 'Error:', msg
    sys.exit()

def _nums_to_str(seq, fmt='%1.3f'):
    return ' '.join([_int_or_float(val, fmt) for val in seq])

def draw_if_interactive():
    pass

def show():
    """
    Show all the figures and enter the gtk mainloop

    This should be the last line of your script
    """
    pass

def new_figure_manager(num, *args):
    thisFig = Figure(*args)
    canvas = FigureCanvasSVG(thisFig)
    manager = FigureManagerSVG(canvas, num)
    return manager

def _rgb_to_hex(rgb):
    rgbhex='#'
    for c in rgb:
        h=hex(int(c*255))[2:]
        if len(h) < 2:
            h='0'+ h
        rgbhex += h
    return rgbhex

fontd = {}
clipd = {}
class RendererSVG(RendererBase):
    def __init__(self, width, height, svgwriter):
        self._svgwriter = svgwriter
        self.width=width
        self.height=height
        self._groupd = {}

    def flipy(self):
        'return true if y small numbers are top for renderer'
        return False

    def get_canvas_width_height(self):
        'return the canvas width and height in display coords'
        return self.width, self.height

    def _draw_rawsvg(self, svg):
        self._svgwriter.write(svg)

    def _get_font(self, prop):
        key = hash(prop)
        font = fontd.get(key)
        if font is None:
            font = AFM(file(fontManager.findfont(prop, fontext='afm')))
            fontd[key] = font
        return font

    def get_text_width_height(self, s, prop, ismath):
        """
        get the width and height in display coords of the string s
        with FontPropertry prop
        """
        font = self._get_font(prop)
        l,b,w,h = font.get_str_bbox(s)

        fontsize = prop.get_size_in_points()
        w *= 0.001*fontsize
        h *= 0.001*fontsize
        return w, h

    def open_group(self, s):
        'open a grouping element with label s'
        group = self._groupd.get(s)
        if group is None:          # This is just to make sure each group gets
            self._groupd[s] = 0    # a unique name.
        if group is not None:
            self._groupd[s] += 1
        svg = '<g id= "' + s + repr(self._groupd[s]) + '\">\n'
        self._draw_rawsvg(svg)

    def close_group(self, s):
        'close a grouping element with label s'
        self._draw_rawsvg('</g>\n')

    def draw_arc(self, gc, rgbFace, x, y, width, height, angle1, angle2):  #for now, draws a circle of diameter width
        """
        Draw a circle at x,y of diameter 'width'
        """
        type = '<circle '
        details = ' cx="%f" \n cy="%f" \n r="%f"\n' % (x,self.height-y,width/2)
        self._draw_svg(type, details, gc, rgbFace)

    def draw_line(self, gc, x1, y1, x2, y2):
        """
        Draw a single line from x1,y1 to x2,y2
        """
        type = '<path '
        details = ' d="M %f,%f L %f,%f" ' % (x1,self.height-y1,x2,self.height-y2)
        self._draw_svg(type, details, gc, None)

    def draw_lines(self, gc, x, y):
        """
        x and y are equal length arrays, draw lines connecting each
        point in x, y
        """

        if len(x)==0: return
        if len(x)!=len(y): error_msg_svg('x and y must be the same length')
        type = '<path '

        y = self.height - y
        details = [' d="M %f,%f' % (x[0], y[0]) ]
        xys = zip(x[1:], y[1:])
        details.extend(['L %f,%f' % tup for tup in xys])
        details.append('" ')
        details = ' '.join(details)
        self._draw_svg(type, details, gc, None)

    def draw_rectangle(self, gc, rgbFace, x, y, width, height):
        if rgbFace is not None:
            rgbhex+='fill:#'
            for c in rgbFace:
                rgbhex += hex(int(c*255))[2:]
        else:
            rbghex='fill:none'
        type = '<rect '
        details = """
        width="%f"
        height="%f"
        x="%f"
        y="%f" """ % (width, height, x, self.height-y-height)
        self._draw_svg(type, details, gc, rgbFace)

    def draw_polygon(self, gc, rgbFace, points):
        type = '<polygon '
        details = '   points = "'
        for point in points:
            details += '%f,%f ' % (point[0],self.height-point[1])
        details += '"'
        self._draw_svg(type, details, gc, rgbFace)

    def draw_point(self, gc, x, y):
        """
        Draw a point at x,y
        """
        self.draw_arc(gc, gc.get_rgb(), x, y, 1, 0, 0, 0)  # result seems to have a hole in it...

    def draw_text(self, gc, x, y, s, prop, angle, ismath):
        """
        draw text
        """

        font = self._get_font(prop)
        
        l,b,w,h = font.get_str_bbox(s)

        fontsize = prop.get_size_in_points()
        l *= 0.001*fontsize
        b *= 0.001*fontsize
        w *= 0.001*fontsize
        h *= 0.001*fontsize

        if angle==90: l,b = -b, l # todo generalize for arb rotations
        x = x-l
        y = self.height - y+b
        thetext = '%s' % s
        fontname = font.get_fontname()
        fontsize = prop.get_size_in_points()
        if angle is not 0:
            transform = '   transform = "translate(%f,%f) rotate(%1.1f) translate(%f,%f)"' % (x,y,-angle,-x,-y) # Inkscape doesn't support rotate(angle x y)
        else: transform = ''
        svg = '<text '

        svg += """style="font-size:%f;stroke-width:1.0000000pt;font-family:%s;"\n   x="%f" \n   y="%f"\n""" % (fontsize,fontname,x,y)
        svg += transform
        svg += '>'+thetext+'</text>\n'
        self._draw_rawsvg(svg)

    def get_svg(self):
        return self._svgwriter.getvalue()

    def finish(self):
        self._svgwriter.write('</svg>')

    def new_gc(self):
        return GraphicsContextSVG()

    def _draw_svg(self, type, details, gc, rgbFace):
        svg=type
        if rgbFace is not None:
            rgbhex='fill:'+_rgb_to_hex(rgbFace)+';'
        else:
            rgbhex='fill:none;'
        style = self._get_gc_props_svg(gc)
        cliprect,id = self._get_gc_clip_svg(gc)
        if id is not None:
            clippath = ' clip-path="url(#%s)"' % id
            if cliprect is not '':
                svg = cliprect + svg
        else: clippath = ''
        svg+=style+rgbhex+ ' "\n'
        svg += details + clippath
        svg += ' />\n'
        self._svgwriter.write(svg)

    def _get_gc_props_svg(self, gc):
        color='stroke:'+_rgb_to_hex(gc.get_rgb())+';'
        linewidth = 'stroke-width:'+repr(gc.get_linewidth())+'pt;'
        join = 'stroke-linejoin:'+gc.get_joinstyle()+';'
        cap = 'stroke-linecap:'+gc.get_capstyle()+';'
        offset, seq = gc.get_dashes()
        if seq is not None:
            dashes = 'stroke-dasharray:'
            for s in seq:
                dashes += '%d ' % s
            dashes += ';'
            dashes += 'stroke-dashoffset:%f;' % offset
        else:
            dashes = ''
        style = 'style="'+color+linewidth+join+cap+dashes
        return style

    def _get_gc_clip_svg(self, gc):
        cliprect = gc.get_clip_rectangle()
        if cliprect is not None:
            key = hash(cliprect)  # See if we've already seen this clip rectangle
            cr = clipd.get(key)
            id=repr(key)
            if cr is None:        # If not, store a new clipPath
                clipd[key] = cliprect
                x=cliprect[0]
                y=self.height-cliprect[1]
                w=cliprect[2]
                h=-cliprect[3]
                box = '<clipPath id="%s"><path d="M %f,%f L %f,%f L %f,%f L %f,%f L %f,%f" /> </clipPath>'% (id,x,y,x+w,y,x+w,y+h,x,y+h,x,y)
                addref = True
                return box, id
            else: return '',id   # If we're using a previously defined clipPath, reference its id
        return '',None

    def new_gc(self):
        """
        Return an instance of a GraphicsContextTemplate
        """
        return GraphicsContextSVG()

class GraphicsContextSVG(GraphicsContextBase):

    def set_linestyle(self, style):
        GraphicsContextBase.set_linestyle(self, style)
        offset, dashes = self._dashd[style]
        self.set_dashes(offset, dashes)

class FigureCanvasSVG(FigureCanvasBase):

    def draw(self):
        pass

    def print_figure(self, filename, dpi=80,
                     facecolor='w', edgecolor='w',
                     orientation='portrait'):
        width, height = self.figure.get_size_inches()

        origfacecolor = self.figure.get_facecolor()
        origedgecolor = self.figure.get_edgecolor()
        self.figure.set_facecolor(facecolor)
        self.figure.set_edgecolor(edgecolor)

        basename, ext = os.path.splitext(filename)
        if not len(ext): filename += '.svg'

        self._svgwriter = StringIO()

        renderer = RendererSVG(width*dpi, height*dpi, self._svgwriter)

        self._svgwriter.write(_svgProlog )

        self.figure.draw(renderer)
        renderer.finish()

        try: fh = file(filename, 'w')
        except IOError:
            error_msg_svg('Could not open %s for writing' % filename)
            return
        print >>fh, renderer.get_svg()

class FigureManagerSVG(FigureManagerBase):
    pass

FigureManager = FigureManagerSVG
error_msg = error_msg_svg

_svgProlog = """<?xml version="1.0" standalone="no"?>
<!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.0//EN"
"http://www.w3.org/TR/2001/REC-SVG-20010904/DTD/svg10.dtd">
<!-- Created with matplotlib (http://matplotlib.sourceforge.net/) -->
<svg
   xmlns="http://www.w3.org/2000/svg"
   xmlns:xlink="http://www.w3.org/1999/xlink"
   version="1.0"
   x="0.0000000"
   y="0.0000000"
   id="svg1">
"""

#encoding=\"UTF-8\"

#width=\"%fin\"
#   height=\"%fin\"

#% (width,height)
